# -*- coding: utf-8 -*-
"""
Script to recreate Figure 02 in manuscript:
Rarest Rainfall extremes will see the greatest relative increase in magnitude 
under future climate change, in Nature Communications Earth and Environment (2022)
by Gaby Joanne Gruendemann, Nick van de Giesen, Lukas Brunner and Ruud van der Ent

@author: ggrundemann
"""

import numpy as np
import xarray as xr
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as mcolors
import string
import cartopy.crs as ccrs
import cartopy.feature as cfeature



#%% specify directories
dir_files  = '' # folder in which the files listed below are stored

maskdry = xr.open_dataset(dir_files + '/mask_dry_areas.nc')
maskdry = maskdry.mask.values
maskdry = maskdry.astype(float)
maskdry[maskdry == 0] = np.nan

#%%
''' 
dqT1w   - difference in T1   for historical vs ssp scenarios, estimated using quantiles, weighted
dmT10w  - difference in T10  for historical vs ssp scenarios, estimated using mev, weighted
dmT100w - difference in T100 for historical vs ssp scenarios, estimated using mev, weighted

hq1   - hatching for difference in T1   for historical vs ssp scenarios, estimating using quantiles
hm10  - hatching for difference in T10  for historical vs ssp scenarios, estimated using mev
hm100 - hatching for difference in T100 for historical vs ssp scenarios, estimated using mev
''' 

dqT1w   = xr.open_dataset(dir_files + '/difT1_quantiles_weighted-mean-med-std-ipr_all_ssps.nc')
dmT10w  = xr.open_dataset(dir_files + '/difT10_MEV_weighted-mean-med-ipr_all_ssps.nc')
dmT100w = xr.open_dataset(dir_files + '/difT100_MEV_weighted-mean-med-ipr_all_ssps.nc')

hq1      = xr.open_dataset(dir_files + '/hatching_difT1_quantiles_all_ssps.nc')
hm10     = xr.open_dataset(dir_files + '/hatching_difT10_MEV_all_ssps.nc')
hm100    = xr.open_dataset(dir_files + '/hatching_difT100_MEV_all_ssps.nc')

plottingmw = [dqT1w, dmT10w, dmT100w]

hatching   = [hq1, hm10, hm100]

#%%
def create_plot_cartopy(plotting, hatching, par='mean', ssp='ssp585', countries=True, 
                        savefig=True, savename='savename',pdf=False,dpi=100):
    fig = plt.figure(figsize=(11/2.54,18/2.54))
    for i in range(0,3):
        print(i)
        ax  = fig.add_subplot(3,1,i+1, projection=ccrs.EckertIV())
        ax.set_global()
        ax.coastlines(linewidth=0.1)
        if countries == True:
            ax.add_feature(cfeature.BORDERS, linewidth=0.1)
        plot  = plotting[i]
        if par == 'median':
            try:
                data = plot['median'].values
            except:
                data = plot['median_'+ssp].values
        elif par == 'mean':
            try:
                data = plot['mean'].values
            except:
                data = plot['mean_'+ssp].values

        data= data * maskdry # added for mask
        
        datae = data[:,rlat:]
        dataw = data[:,:rlat]
        dataew = np.hstack((datae, dataw))
        dataplot = np.flipud(dataew)
        
        
        hatch   = hatching[i]
        mhatch  = hatch['hatching_'+ssp].values
        mhatchplot  = mhatch * maskdry
        
        mp = np.ma.masked_not_equal(mhatchplot, 1)
        ax.imshow(np.ones((720, 1440)), transform=ccrs.PlateCarree(), cmap=cm.gray, vmin=0, vmax=2)

        m = ax.imshow(dataplot, transform=ccrs.PlateCarree(), cmap=cmap, vmin=vmin, vmax=vmax, norm=norm)
        ax.contourf(lons, lats, mp, transform=ccrs.PlateCarree(), colors='none',hatches=['........']) #lonsm, lats, 
        plt.tight_layout()
        ax.set_title(titles[i], fontsize=8)
        plt.gcf().text(0.05, locsy[i], string.ascii_lowercase[i]+')',fontsize=8)
    cax = fig.add_axes([0.05, 0, 0.9, 0.01]) # [left, bottom, width, height]
    cb = plt.colorbar(m, cax=cax,orientation='horizontal',extend='both',aspect=0.4,ticks=lvls)
    cb.ax.tick_params(labelsize=8) 
    cb.set_label('change (%)',fontsize=8)
    plt.tight_layout()
    if savefig == True:
        plt.savefig(savename + '.png', dpi=dpi, bbox_inches="tight")
        if pdf == True:
            plt.savefig(savename + '.pdf', dpi=dpi, bbox_inches="tight")
    return plt.show()

#%%
vmin = -50
vmax = 100
ssp  = 'ssp585'
rlat = 720
rlon = 1440
mpl.rcParams['hatch.linewidth'] = 0.1

f            = lambda x: ((x+180) % 360) - 180
lat          = dqT1w.lat.values
lon          = dqT1w.lon.values
lon          = f(lon)
ind          = np.argsort(lon)
lonm         = lon[ind]
lonsm, lats  = np.meshgrid(lonm, dqT1w.lat)
lons, lats   = np.meshgrid(dqT1w.lon, dqT1w.lat)


#%% plot 4 subplots, hatching
lvls = np.arange(vmin,vmax+1,10)
norm = mcolors.TwoSlopeNorm(vmin=vmin, vcenter=0., vmax=vmax)
cmap = cm.BrBG

locsy = [0.96,0.637,0.315]

titles = ['1-year return level','10-year return level','100-year return level']

savename = 'Fig_02'
create_plot_cartopy(plottingmw, hatching, par='mean',ssp=ssp,countries=True,savefig=True,savename=savename,pdf=True,dpi=300)

