# -*- coding: utf-8 -*-
"""
Script to recreate Figure 3 in manuscript:
Rarest Rainfall extremes will see the greatest relative increase in magnitude 
under future climate change, in Nature Communications Earth and Environment (2022)
by Gaby Joanne Gruendemann, Nick van de Giesen, Lukas Brunner and Ruud van der Ent

@author: ggrundemann
"""
#%% load packages
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.cm as cm

#%% specify directories
dir_files = '' # folder in which the files listed below are stored

#%%
''' 
dfm contains scatterplot data for T10 and T100 using MEV. T1 is retrieved from the quantiles themself
dfg contains scatterplot data for T10 and T100 using GEV. T1 is retrieved from the quantiles themself
dfq contains scatterplot data for T0.3, T3 and T30 using the quantiles themself
note that only dfm is used in the paper, dfg and dfq are for the supplementary information
'''

dfm = pd.read_csv (dir_files +'\data_figure_03_mev.csv', index_col=0)
dfg = pd.read_csv (dir_files +'\data_figure_03_gev.csv', index_col=0)
dfq = pd.read_csv (dir_files +'\data_figure_03_quantiles.csv', index_col=0)

#%% functions
def sortby(df, scen, itr, descending=True,par='med'):
    dfsort  = df.copy()
    dfsort  = dfsort[(dfsort['scenario'] == scen) & (dfsort['itr'] == itr)]
    columns = ['modelid',par,'model']
    v = dfsort[columns].values
    a = np.argsort(-1*v[:,1])
    if descending == False:
        a = np.argsort(v[:,1])
    return a


def scatterplot_figure(df, trs, sortscen=None, dfm=None, ylim=None, fontsize=8, 
                       markers=None, dpi=300, ssave=None, title=None):
    dfp = df.copy()
    scenlab = ['SSP1-2.6','SSP2-4.5','SSP3-7.0','SSP5-8.5']
    j=0

    fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(23/2.54,18/2.54),sharex=True,sharey=True)
    for ax, scen in zip(axes.flatten(), ssps):
        for i in range(0,len(trs)):
            if sortscen == 'cellsize':
                dfp = dfp.sort_values(['cells','scenario'])
            if sortscen == 'mev585100':
                dfsel       = df.copy()
                mev585100   = sortby(dfm, 'ssp585', 100)
                dfsel       = dfsel[(dfsel['scenario'] == scen) & (dfsel['itr'] == trs[i])]
                columns     = ['modelid','med','model','q1','q3','itr','scenario']
                vsel        = dfsel[columns].values
                dfp         = pd.DataFrame(vsel[mev585100], columns=columns)
            dfp['q1']   = dfp['q1'].astype(float)
            dfp['q3']   = dfp['q3'].astype(float)
            flow  = dfp[(dfp['itr'] == trs[i]) & (dfp['scenario'] == scen)].q1
            fhigh = dfp[(dfp['itr'] == trs[i]) & (dfp['scenario'] == scen)].q3


            if markers==None:
                mrk='o'
            else:
                mrk=markers[i]

            ax.plot(        dfp.model.unique(), dfp[(dfp['itr'] == trs[i]) & (dfp['scenario'] == scen)].med, color=cols[i], label='T'+str(trs[i]),marker=mrk,linestyle='--')
            ax.fill_between(dfp.model.unique(), flow, fhigh, facecolor=cols[i], alpha=0.2, edgecolor=cols[i],linewidth=1)
        ax.tick_params(axis='x',rotation=90)
        ax.tick_params(axis='both',which='major',labelsize=fontsize)
        ax.grid(linewidth=0.5,linestyle='--',color='grey',axis='y')
        ax.set_xlim(-.5,24.5)
        ax.legend(loc='upper right',fontsize=fontsize)
        if ylim != None:
            ax.set_ylim(ylim)
        ax.set_title(scenlab[j],fontsize=fontsize)
        j += 1
    if title != None:
        plt.suptitle(title, fontsize=fontsize)
    axes[0,0].set_ylabel('change (%)',fontsize=fontsize)
    axes[1,0].set_ylabel('change (%)',fontsize=fontsize)
    plt.tight_layout()
    plt.tight_layout()
    # plt.savefig(ssave + '.pdf', dpi=dpi)
    return plt.show()


#%% parameters for saving, color maps
savefigs     = False
closefigs    = False
cmap         = cm.viridis_r
ssps         = ['ssp126','ssp245','ssp370','ssp585']
scenarios    = ['his','ssp126','ssp245','ssp370','ssp585']

dpi=300
cols = ['#0077b6','#f37738','g']

''' Fig_03 - scatterplot MEV '''
scatterplot_figure(dfm, trs=[100,10,1], sortscen='mev585100', dfm=dfm, ylim=(-6,70), fontsize=8, markers=['s','o','^'],ssave='Fig_03')

# ''' Supplementary figure - scatterplot GEV ''' 
# scatterplot_figure(dfg, trs=[100,10,1], sortscen='mev585100', dfm=dfm, ylim=(-15, 100), fontsize=8, markers=['s','o','^'],ssave='Fig03_GEV',       title='GEV', )

# ''' Supplementary figure - scatterplot quantiles '''
# scatterplot_figure(dfq, trs=[30,3,0.3], sortscen='mev585100', dfm=dfm, ylim=(-15,100), fontsize=8, markers=['s','o','^'],ssave='Fig03_quantiles', title='"observed" quantiles')

#%% values - useful for refering to in the paper
# itr  = 100
# scen = 'ssp126'
# dforig = dfm.copy()

# list_med = []
# list_q1  = []
# list_q3  = []
# list_m1  = []
# list_m3  = []
# list_sc  = []
# list_tr  = []
# list_mx  = []
# list_mn  = []
# for itr in [10, 100, 1000]:
#     for scen in ['ssp126', 'ssp245', 'ssp370', 'ssp585']:
#         s = dforig[(dforig['itr'] == itr) & (dforig['scenario'] == scen)].describe(include='all')
#         list_med.append(s['med'].iloc[8])
#         list_q1.append(s['q1'].iloc[8])
#         list_q3.append(s['q3'].iloc[8])
#         list_mx.append(s['med'].iloc[-1])
#         list_mn.append(s['med'].iloc[6])
#         list_m1.append(s['med'].iloc[7])
#         list_m3.append(s['med'].iloc[9])
#         list_sc.append(scen)
#         list_tr.append(itr)
# df = pd.DataFrame()
# df['scen']    = list_sc
# df['tr']      = list_tr
# df['med_min'] = list_mn
# df['med_q1']  = list_m1
# df['median']  = list_med
# df['med_max'] = list_mx
# df['med_q3']  = list_m3
# df['q1']      = list_q1
# df['q3']      = list_q3

